@@ -313,11 +313,11 @@ fy_atom_iter_line_analyze(struct fy_atom_iter *iter, struct fy_atom_iter_line_in
 	s = line_start;
 	e = line_start + len;
 
-	is_block = atom->style == FYAS_LITERAL || atom->style == FYAS_FOLDED;
+	is_block = fy_atom_style_is_block(atom->style);
 
 	/* short circuit non multiline, non ws atoms */
 	if ((atom->direct_output && !atom->has_lb && !atom->has_ws) ||
-	    atom->style == FYAS_DOUBLE_QUOTED_MANUAL) {
+	    (atom->style & FYAS_MANUAL_MARK)) {
 		li->start = s;
 		li->end = e;
 		li->nws_start = s;
@@ -451,7 +451,7 @@ fy_atom_iter_line_analyze(struct fy_atom_iter *iter, struct fy_atom_iter_line_in
 			fprintf(stderr, "%s:%d tab col=%d chomp=%d\n", __FILE__, __LINE__, col, (int)(li->chomp_start - li->start));
 #endif
 
-			if (fy_atom_style_is_block(atom->style) && col >= (int)(li->chomp_start - li->start)) {
+			if (fy_atom_style_is_block(atom->style) && li->chomp_start && col >= (int)(li->chomp_start - li->start)) {
 #if defined(DEBUG_CHUNK)
 				fprintf(stderr, "%s:%d tab col=%d chomp=%d\n", __FILE__, __LINE__, col, (int)(li->chomp_start - li->start));
 #endif
@@ -711,6 +711,9 @@ fy_atom_iter_line(struct fy_atom_iter *iter)
 
 	/* scan next line (special handling for '\r\n') */
 	ss = li->end;
+	if (!ss)
+		return NULL;
+
 	if (ss < iter->e) {
 		if (*ss == '\r' && (ss + 1) < iter->e && ss[1] == '\n')
 			ss += 2;
@@ -727,7 +730,7 @@ fy_atom_iter_line(struct fy_atom_iter *iter)
 		nli = NULL;
 
 	/* for quoted, output the white space start */
-	if (atom->style == FYAS_SINGLE_QUOTED || atom->style == FYAS_DOUBLE_QUOTED) {
+	if (fy_atom_style_is_quoted(atom->style)) {
 		li->s = li->first ? li->start : li->nws_start;
 		li->e = li->last ? li->end : li->nws_end;
 
@@ -735,7 +738,7 @@ fy_atom_iter_line(struct fy_atom_iter *iter)
 		if (li->empty && li->first && li->last && !iter->single_line)
 			li->s = li->e;
 
-	} else if (atom->style == FYAS_LITERAL || atom->style == FYAS_FOLDED) {
+	} else if (fy_atom_style_is_block(atom->style)) {
 		li->s = li->chomp_start;
 		li->e = li->end;
 		if (li->empty && li->first && li->last)
@@ -768,11 +771,6 @@ fy_atom_iter_line(struct fy_atom_iter *iter)
 		li->need_sep = !li->need_nl && nli && !nli->empty;
 		break;
 
-	case FYAS_DOUBLE_QUOTED_MANUAL:
-		li->need_nl = false;
-		li->need_sep = false;
-		break;
-
 	case FYAS_COMMENT:
 		li->need_nl = !li->final;
 		li->need_sep = false;
@@ -811,7 +809,37 @@ fy_atom_iter_line(struct fy_atom_iter *iter)
 			break;
 		li->need_sep = nli && !nli->indented && !nli->empty;
 		break;
+
+	case FYAS_PLAIN_MANUAL:
+		li->need_nl = false;
+		li->need_sep = false;
+		break;
+
+	case FYAS_SINGLE_QUOTED_MANUAL:
+		li->need_nl = false;
+		li->need_sep = false;
+		break;
+
+	case FYAS_DOUBLE_QUOTED_MANUAL:
+		li->need_nl = false;
+		li->need_sep = false;
+		break;
+
+	case FYAS_LITERAL_MANUAL:
+		li->need_nl = false;
+		li->need_sep = false;
+		break;
+
+	case FYAS_FOLDED_MANUAL:
+		li->need_nl = false;
+		li->need_sep = false;
+		break;
+
 	default:
+		if (atom->style & FYAS_MANUAL_MARK) {
+			li->need_nl = false;
+			li->need_sep = false;
+		}
 		break;
 	}
 
@@ -876,11 +904,11 @@ fy_atom_iter_format(struct fy_atom_iter *iter)
 
 			s = t;
 			/* next character single quote too */
-			if ((e - s) >= 2 && s[1] == '\'')
+			if ((e - s) >= 2 && s[1] == '\'') {
 				fy_atom_iter_add_chunk(iter, s, 1);
-
-			/* skip over this single quote char */
-			s++;
+				s += 2;	/* skip over the two single quotes */
+			} else
+				s++;	/* skip over this single quote char */
 		}
 		break;
 
@@ -950,12 +978,18 @@ fy_atom_iter_format(struct fy_atom_iter *iter)
 		}
 		break;
 
+	case FYAS_PLAIN_MANUAL:
+	case FYAS_SINGLE_QUOTED_MANUAL:
 	case FYAS_DOUBLE_QUOTED_MANUAL:
+	case FYAS_LITERAL_MANUAL:
+	case FYAS_FOLDED_MANUAL:
 		/* manual scalar just goes out */
-		ret = fy_atom_iter_add_chunk(iter, s, e - s);
-		if (ret)
-			goto out;
-		s = e;
+		if (s < e) {
+			ret = fy_atom_iter_add_chunk(iter, s, e - s);
+			if (ret)
+				goto out;
+			s = e;
+		}
 		break;
 
 	default:
@@ -1216,9 +1250,11 @@ const char *fy_atom_format_text(struct fy_atom *atom, char *buf, size_t maxsz)
 	fy_atom_iter_start(atom, &iter);
 	ic = NULL;
 	while ((ic = fy_atom_iter_chunk_next(&iter, ic, &ret)) != NULL) {
-		/* must fit */
-		if ((size_t)(e - s) < ic->len)
-			return NULL;
+		/* must fit (something is really off) */
+		if ((size_t)(e - s) < ic->len) {
+			ret = -1;
+			break;
+		}
 		memcpy(s, ic->str, ic->len);
 		s += ic->len;
 	}
@@ -1852,7 +1888,7 @@ fy_atom_raw_line_iter_next(struct fy_atom_raw_line_iter *iter)
 
 		s += w;
 		/* special case for MSDOS */
-		if (c == '\r' && (s < iter->ie && s[1] == '\n'))
+		if (c == '\r' && (s + 1 < iter->ie && s[1] == '\n'))
 			s++;
 		/* len_lb includes the lb */
 		l->line_len_lb = (size_t)(s - l->line_start);
@@ -1932,3 +1968,265 @@ const char *fy_atom_lines_containing(struct fy_atom *atom, size_t *lenp)
 	*lenp = end - start;
 	return start;
 }
+
+unsigned int
+fy_atom_text_analyze(struct fy_atom *handle, enum fy_atom_style style, int *maxspanp, int *maxcolp)
+{
+	unsigned int flags = 0;
+	int c, cn, cnn, cp, col;
+	uint8_t col0si, col0ei;	/* mask for --- ... at indent 0 */
+	int span, maxspan, maxcol;
+	struct fy_atom_iter iter;
+
+	if (maxspanp)
+		*maxspanp = 0;
+	if (maxcolp)
+		*maxcolp = 0;
+
+	flags = FYTTAF_TEXT_TOKEN;
+
+	/* hardwired and fast for regular plain scalars */
+	if (style == FYAS_PLAIN && handle->storage_hint_valid &&
+	    handle->direct_output && !handle->high_ascii &&
+	    !handle->has_lb && !handle->has_ws && !handle->empty) {
+
+		flags |= FYTTAF_DIRECT_OUTPUT;
+
+		maxcol = (int)handle->storage_hint;
+		maxspan = maxcol - 1;
+		flags |=
+			FYTTAF_DIRECT_OUTPUT |
+			FYTTAF_CAN_BE_SIMPLE_KEY |
+			FYTTAF_CAN_BE_PLAIN |
+			FYTTAF_CAN_BE_SINGLE_QUOTED |
+			FYTTAF_CAN_BE_DOUBLE_QUOTED |
+			FYTTAF_CAN_BE_LITERAL |
+			FYTTAF_CAN_BE_PLAIN_FLOW |
+			FYTTAF_CAN_BE_UNQUOTED_PATH_KEY;
+
+		goto done;
+	}
+
+	/* can this token be a simple key initial condition */
+	if (!fy_atom_style_is_block(style) && style != FYAS_URI)
+		flags |= FYTTAF_CAN_BE_SIMPLE_KEY;
+
+	/* can this token be directly output initial condition */
+	if (!fy_atom_style_is_block(style))
+		flags |= FYTTAF_DIRECT_OUTPUT;
+
+	fy_atom_iter_start(handle, &iter);
+
+	col = 0;
+	maxcol = 0;
+	maxspan = 0;
+	span = 0;
+
+	/* get first character */
+	cn = fy_atom_iter_utf8_get(&iter);
+	if (cn < 0) {
+		/* empty? */
+		flags |= FYTTAF_SIZE0 | FYTTAF_EMPTY | FYTTAF_CAN_BE_UNQUOTED_PATH_KEY | FYTTAF_CAN_BE_SIMPLE_KEY;
+		goto out;
+	}
+
+	flags |= FYTTAF_CAN_BE_PLAIN |
+		 FYTTAF_CAN_BE_SINGLE_QUOTED |
+		 FYTTAF_CAN_BE_DOUBLE_QUOTED |
+		 FYTTAF_CAN_BE_LITERAL |
+		 FYTTAF_CAN_BE_FOLDED |
+		 FYTTAF_CAN_BE_PLAIN_FLOW |
+		 FYTTAF_CAN_BE_UNQUOTED_PATH_KEY |
+		 FYTTAF_ALL_WS_LB |
+		 FYTTAF_ALL_PRINT_ASCII |
+		 FYTTAF_EMPTY;
+
+	col0si = col0ei = 0;
+
+	/* disable folded right off the bat, it's a pain */
+	flags &= ~FYTTAF_CAN_BE_FOLDED;
+
+	/* plain scalars can't start with any indicator (or space/lb) */
+	if ((flags & (FYTTAF_CAN_BE_PLAIN | FYTTAF_CAN_BE_PLAIN_FLOW))) {
+		if (fy_is_start_indicator(cn) || fy_atom_is_lb(handle, cn) || fy_is_ws(cn))
+			flags &= ~(FYTTAF_CAN_BE_PLAIN | FYTTAF_CAN_BE_PLAIN_FLOW);
+	}
+
+	/* plain scalars in flow mode can't start with a flow indicator */
+	if ((flags & FYTTAF_CAN_BE_PLAIN_FLOW) &&
+		fy_is_flow_indicator(cn))
+		flags &= ~FYTTAF_CAN_BE_PLAIN_FLOW;
+
+	if ((flags & (FYTTAF_CAN_BE_PLAIN | FYTTAF_CAN_BE_PLAIN_FLOW))) {
+		cnn = fy_atom_iter_utf8_peek(&iter);
+		if (fy_is_blankz_m(cnn, fy_atom_lb_mode(handle)) && fy_is_indicator_before_space(cn))
+			flags &= ~(FYTTAF_CAN_BE_PLAIN | FYTTAF_CAN_BE_PLAIN_FLOW);
+	}
+
+	/* plain unquoted path keys can only start with [a-zA-Z_] */
+	if ((flags & FYTTAF_CAN_BE_UNQUOTED_PATH_KEY) &&
+		!fy_is_first_alpha(cn))
+		flags &= ~FYTTAF_CAN_BE_UNQUOTED_PATH_KEY;
+
+	/* if it starts with white space can't be a plain */
+	if (fy_is_ws(cn)) {
+		flags |= FYTTAF_HAS_START_WS;
+		flags &= ~(FYTTAF_CAN_BE_PLAIN |
+			   FYTTAF_CAN_BE_PLAIN_FLOW);
+	}
+
+	cp = -1;
+	for (c = cn; c >= 0; cp = c, c = cn) {
+
+		if (col <= 2) {
+			if (cn == '-') {
+				col0si |= (uint8_t)1 << col;
+				if (col0si == 7)
+					flags |= FYTTAF_HAS_START_IND | FYTTAF_QUOTE_AT_0;
+			} else if (cn == '.') {
+				col0ei |= (uint8_t)1 << col;
+				if (col0ei == 7)
+					flags |= FYTTAF_HAS_END_IND | FYTTAF_QUOTE_AT_0;
+			}
+		}
+
+		/* can be -1 on end */
+		cn = fy_atom_iter_utf8_get(&iter);
+
+		/* zero can't be output, only in double quoted mode */
+		if (c == 0) {
+			flags &= ~(FYTTAF_DIRECT_OUTPUT |
+				   FYTTAF_CAN_BE_PLAIN |
+				   FYTTAF_CAN_BE_SINGLE_QUOTED |
+				   FYTTAF_CAN_BE_LITERAL |
+				   FYTTAF_CAN_BE_FOLDED |
+				   FYTTAF_CAN_BE_PLAIN_FLOW |
+				   FYTTAF_CAN_BE_UNQUOTED_PATH_KEY |
+				   FYTTAF_ALL_WS_LB |
+				   FYTTAF_ALL_PRINT_ASCII);
+			flags |= FYTTAF_CAN_BE_DOUBLE_QUOTED;
+			flags &= ~FYTTAF_EMPTY;
+			flags |= FYTTAF_HAS_ZERO;
+		} else if (fy_is_ws(c)) {
+
+			flags |= FYTTAF_HAS_WS;
+			if (fy_is_ws(cn)) {
+				flags |= FYTTAF_HAS_CONSECUTIVE_WS;
+
+				/* on a manual ' style we can't have linebreak and then consecutive ws */
+				if (style == FYAS_SINGLE_QUOTED_MANUAL && fy_atom_is_lb(handle, cp))
+					flags &= ~FYTTAF_CAN_BE_SINGLE_QUOTED;
+			}
+
+			/* non printable ascii */
+			flags &= ~FYTTAF_ALL_PRINT_ASCII;
+
+		} else if (fy_atom_is_lb(handle, c)) {
+
+			flags |= FYTTAF_HAS_LB;
+			if (fy_atom_is_lb(handle, cn))
+				flags |= FYTTAF_HAS_CONSECUTIVE_LB;
+
+			/* non printable ascii */
+			flags &= ~FYTTAF_ALL_PRINT_ASCII;
+			flags &= ~FYTTAF_EMPTY;
+
+			/* only non linebreaks can be simple keys */
+			flags &= ~FYTTAF_CAN_BE_SIMPLE_KEY;
+
+			/* anything with linebreaks, can't be direct */
+			flags &= ~FYTTAF_DIRECT_OUTPUT;
+
+			if (c != '\n')
+				flags |= FYTTAF_HAS_NON_NL_LB;
+		} else {
+			flags &= ~FYTTAF_EMPTY;
+			flags &= ~FYTTAF_ALL_WS_LB;
+			/* out of our comfort zone */
+			if (c < '!' || c > '~')
+				flags &= ~FYTTAF_ALL_PRINT_ASCII;
+		}
+
+		if ((flags & FYTTAF_CAN_BE_UNQUOTED_PATH_KEY) && !fy_is_alnum(c))
+			flags &= ~FYTTAF_CAN_BE_UNQUOTED_PATH_KEY;
+
+		/* illegal plain combination */
+		if ((flags & FYTTAF_CAN_BE_PLAIN) &&
+			((c == ':' && fy_is_blankz_m(cn, fy_atom_lb_mode(handle))) ||
+			 (fy_is_blankz_m(c, fy_atom_lb_mode(handle)) && cn == '#') ||
+			 (cp < 0 && c == '#' && cn < 0) ||
+			 !fy_is_print(c))) {
+			flags &= ~(FYTTAF_CAN_BE_PLAIN |
+				   FYTTAF_CAN_BE_PLAIN_FLOW);
+		}
+
+		/* illegal plain flow combination */
+		if ((flags & FYTTAF_CAN_BE_PLAIN_FLOW) &&
+			(fy_is_flow_indicator(c) || (c == ':' && fy_is_flow_indicator(cn))))
+			flags &= ~FYTTAF_CAN_BE_PLAIN_FLOW;
+
+		/* non printable characters, turn off these styles */
+		if (!fy_is_print(c)) {
+			flags &= ~(FYTTAF_CAN_BE_SINGLE_QUOTED | FYTTAF_CAN_BE_LITERAL |
+				   FYTTAF_CAN_BE_FOLDED);
+			flags |= FYTTAF_HAS_NON_PRINT;
+		}
+
+		/* if there's an escape, it can't be direct */
+		if ((flags & FYTTAF_DIRECT_OUTPUT) &&
+		    ((style == FYAS_URI && c == '%') ||
+		     (style == FYAS_SINGLE_QUOTED && c == '\'') ||
+		     (style == FYAS_DOUBLE_QUOTED && c == '\\')))
+			flags &= ~FYTTAF_DIRECT_OUTPUT;
+
+		if (cn < 0 || !c || fy_atom_is_lb(handle, c) || fy_is_ws(c)) {
+			if (span > maxspan)
+				maxspan = span;
+			span = 0;
+		} else
+			span++;
+
+		if (fy_atom_is_lb(handle, c)) {
+			if (col > maxcol)
+				maxcol = col;
+			col = 0;
+			col0si = col0ei = 0;
+		} else
+			col++;
+
+		if (fy_is_any_lb(c))
+			flags |= FYTTAF_HAS_ANY_LB;
+
+		/* last character */
+		if (cn < 0) {
+			/* if ends with whitespace or linebreak, or : can't be plain */
+			if (fy_is_ws(c) || fy_atom_is_lb(handle, c) || c == ':') {
+				flags &= ~(FYTTAF_CAN_BE_PLAIN |
+					   FYTTAF_CAN_BE_PLAIN_FLOW);
+				if (c == ':')
+					flags |= FYTTAF_ENDS_WITH_COLON;
+			}
+		}
+	}
+
+	if (col > maxcol)
+		maxcol = col;
+	if (span > maxspan)
+		maxspan = span;
+
+out:
+	/* if it's got nothing, it can be anything */
+	if (flags & FYTTAF_SIZE0)
+		flags |= FYTTAF_CAN_BE_PLAIN | FYTTAF_CAN_BE_PLAIN_FLOW |
+			 FYTTAF_CAN_BE_SINGLE_QUOTED | FYTTAF_CAN_BE_DOUBLE_QUOTED |
+			 FYTTAF_CAN_BE_LITERAL |  FYTTAF_CAN_BE_FOLDED;
+
+	fy_atom_iter_finish(&iter);
+
+done:
+	if (maxspanp)
+		*maxspanp = maxspan;
+	if (maxcolp)
+		*maxcolp = maxcol;
+	return flags;
+}
