@@ -169,6 +169,11 @@ void fy_parser_event_free(struct fy_parser *fyp, struct fy_event *fye)
 	if (!fyp || !fye)
 		return;
 
+	if (fy_reader_generates_events(fyp->reader)) {
+		fy_reader_event_free(fyp->reader, fye);
+		return;
+	}
+
 	fyep = container_of(fye, struct fy_eventp, e);
 
 	fy_parse_eventp_recycle(fyp, fyep);
@@ -337,6 +342,7 @@ fy_eventp_vcreate_internal(struct fy_eventp_list *recycled_list, struct fy_diag
 	case FYET_ALIAS:
 
 		if (type == FYET_SCALAR) {
+			fye->scalar.value = fye->scalar.anchor = fye->scalar.tag = NULL;
 			sstyle = va_arg(ap, enum fy_scalar_style);
 			value = va_arg(ap, const char *);
 			len = va_arg(ap, size_t);
@@ -347,6 +353,7 @@ fy_eventp_vcreate_internal(struct fy_eventp_list *recycled_list, struct fy_diag
 			if (len == FY_NT)
 				len = strlen(value);
 		} else {
+			fye->alias.anchor = NULL;
 			sstyle = FYSS_PLAIN;
 			value = va_arg(ap, const char *);
 			if (!value) {
@@ -367,7 +374,7 @@ fy_eventp_vcreate_internal(struct fy_eventp_list *recycled_list, struct fy_diag
 		memcpy(data, value, len);
 		/* always NULL terminate */
 		data[len] = '\0';
-		fyi = fy_input_from_malloc_data(data, len, &handle, sstyle == FYSS_PLAIN);
+		fyi = fy_input_from_malloc_data_styled(data, len, &handle, sstyle);
 		if (!fyi) {
 			fy_error(diag, "fy_input_from_malloc_data() failed\n");
 			goto err_out;
@@ -417,7 +424,7 @@ fy_eventp_vcreate_internal(struct fy_eventp_list *recycled_list, struct fy_diag
 		/* always NULL terminate */
 		data[len] = '\0';
 
-		fyi = fy_input_from_malloc_data(data, len, &handle, true);
+		fyi = fy_input_from_malloc_data(data, len, &handle, false);
 		if (!fyi) {
 			fy_error(diag, "fy_input_from_malloc_data() failed\n");
 			goto err_out;
@@ -465,7 +472,7 @@ fy_eventp_vcreate_internal(struct fy_eventp_list *recycled_list, struct fy_diag
 			goto err_out;
 		}
 
-		fyi = fy_input_from_malloc_data(data, len, &handle, true);
+		fyi = fy_input_from_malloc_data(data, len, &handle, false);
 		if (!fyi)
 			goto err_out;
 		data = NULL;
@@ -791,7 +798,8 @@ struct fy_token *fy_event_get_tag_token(struct fy_event *fye)
 	return NULL;
 }
 
-const struct fy_mark *fy_event_start_mark(struct fy_event *fye)
+const struct fy_mark *
+fy_event_start_mark(struct fy_event *fye)
 {
 	if (!fye)
 		return NULL;
@@ -835,7 +843,8 @@ const struct fy_mark *fy_event_start_mark(struct fy_event *fye)
 	return NULL;
 }
 
-const struct fy_mark *fy_event_end_mark(struct fy_event *fye)
+const struct fy_mark *
+fy_event_end_mark(struct fy_event *fye)
 {
 	if (!fye)
 		return NULL;
@@ -879,6 +888,103 @@ const struct fy_mark *fy_event_end_mark(struct fy_event *fye)
 	return NULL;
 }
 
+const struct fy_mark *
+fy_event_style_start_mark(struct fy_event *fye)
+{
+	if (!fye)
+		return NULL;
+
+	switch (fye->type) {
+	case FYET_NONE:
+		break;
+
+	case FYET_STREAM_START:
+		return fy_token_style_start_mark(fye->stream_start.stream_start);
+
+	case FYET_STREAM_END:
+		return fy_token_style_start_mark(fye->stream_end.stream_end);
+
+	case FYET_DOCUMENT_START:
+		return fy_token_style_start_mark(fye->document_start.document_start);
+
+	case FYET_DOCUMENT_END:
+		return fy_token_style_start_mark(fye->document_end.document_end);
+
+	case FYET_MAPPING_START:
+		if (fye->mapping_start.tag)
+			return fy_token_style_start_mark(fye->mapping_start.tag);
+		return fy_token_style_start_mark(fye->mapping_start.mapping_start);
+
+	case FYET_MAPPING_END:
+		return fy_token_style_start_mark(fye->mapping_end.mapping_end);
+
+	case FYET_SEQUENCE_START:
+		if (fye->sequence_start.tag)
+			return fy_token_style_start_mark(fye->sequence_start.tag);
+		return fy_token_style_start_mark(fye->sequence_start.sequence_start);
+
+	case FYET_SEQUENCE_END:
+		return fy_token_style_start_mark(fye->sequence_end.sequence_end);
+
+	case FYET_SCALAR:
+		if (fye->scalar.tag)
+			return fy_token_style_start_mark(fye->scalar.tag);
+		return fy_token_style_start_mark(fye->scalar.value);
+
+	case FYET_ALIAS:
+		return fy_token_style_start_mark(fye->alias.anchor);
+
+	}
+
+	return NULL;
+}
+
+const struct fy_mark *
+fy_event_style_end_mark(struct fy_event *fye)
+{
+	if (!fye)
+		return NULL;
+
+	switch (fye->type) {
+	case FYET_NONE:
+		break;
+
+	case FYET_STREAM_START:
+		return fy_token_style_end_mark(fye->stream_start.stream_start);
+
+	case FYET_STREAM_END:
+		return fy_token_style_end_mark(fye->stream_end.stream_end);
+
+	case FYET_DOCUMENT_START:
+		return fy_token_style_end_mark(fye->document_start.document_start);
+
+	case FYET_DOCUMENT_END:
+		return fy_token_style_end_mark(fye->document_end.document_end);
+
+	case FYET_MAPPING_START:
+		return fy_token_style_end_mark(fye->mapping_start.mapping_start);
+
+	case FYET_MAPPING_END:
+		return fy_token_style_end_mark(fye->mapping_end.mapping_end);
+
+	case FYET_SEQUENCE_START:
+		return fy_token_style_end_mark(fye->sequence_start.sequence_start);
+
+	case FYET_SEQUENCE_END:
+		return fy_token_style_end_mark(fye->sequence_end.sequence_end);
+
+	case FYET_SCALAR:
+		return fy_token_style_end_mark(fye->scalar.value);
+
+	case FYET_ALIAS:
+		return fy_token_style_end_mark(fye->alias.anchor);
+
+	}
+
+	return NULL;
+}
+
+
 enum fy_node_style
 fy_event_get_node_style(struct fy_event *fye)
 {
@@ -1012,6 +1118,229 @@ fy_document_start_event_version(struct fy_event *fye)
 	return fy_document_state_version(fye->document_start.document_state);
 }
 
+char *fy_event_to_string(struct fy_event *fye)
+{
+	struct fy_memstream *fyms = NULL;
+	FILE *fp = NULL;
+	char *mbuf = NULL;
+	const char *anchor = NULL;
+	const char *tag = NULL;
+	const char *text = NULL;
+	const char *alias = NULL;
+	size_t msize, anchor_len = 0, tag_len = 0, text_len = 0, alias_len = 0;
+	enum fy_scalar_style style;
+	const uint8_t *p;
+	int i, c, w;
+
+	if (!fye)
+		return NULL;
+
+	fyms = fy_memstream_open(&fp);
+	if (!fyms)
+		return NULL;
+
+	/* event type */
+	switch (fye->type) {
+	case FYET_NONE:
+		fprintf(fp, "???");
+		break;
+	case FYET_STREAM_START:
+		fprintf(fp, "+STR");
+		break;
+	case FYET_STREAM_END:
+		fprintf(fp, "-STR");
+		break;
+	case FYET_DOCUMENT_START:
+		fprintf(fp, "+DOC");
+		break;
+	case FYET_DOCUMENT_END:
+		fprintf(fp, "-DOC");
+		break;
+	case FYET_MAPPING_START:
+		fprintf(fp, "+MAP");
+		if (fye->mapping_start.anchor)
+			anchor = fy_token_get_text(fye->mapping_start.anchor, &anchor_len);
+		if (fye->mapping_start.tag)
+			tag = fy_token_get_text(fye->mapping_start.tag, &tag_len);
+		if (fy_event_get_node_style(fye) == FYNS_FLOW)
+			fprintf(fp, " {}");
+		break;
+	case FYET_MAPPING_END:
+		fprintf(fp, "-MAP");
+		break;
+	case FYET_SEQUENCE_START:
+		fprintf(fp, "+SEQ");
+		if (fye->sequence_start.anchor)
+			anchor = fy_token_get_text(fye->sequence_start.anchor, &anchor_len);
+		if (fye->sequence_start.tag)
+			tag = fy_token_get_text(fye->sequence_start.tag, &tag_len);
+		if (fy_event_get_node_style(fye) == FYNS_FLOW)
+			fprintf(fp, " []");
+		break;
+	case FYET_SEQUENCE_END:
+		fprintf(fp, "-SEQ");
+		break;
+	case FYET_SCALAR:
+		fprintf(fp, "=VAL");
+		if (fye->scalar.anchor)
+			anchor = fy_token_get_text(fye->scalar.anchor, &anchor_len);
+		if (fye->scalar.tag)
+			tag = fy_token_get_text(fye->scalar.tag, &tag_len);
+		break;
+	case FYET_ALIAS:
+		fprintf(fp, "=ALI");
+		break;
+	default:
+		break;
+	}
+
+	/* (position) anchor and tag */
+	if (anchor)
+		fprintf(fp, " &%.*s", (int)anchor_len, anchor);
+	if (tag)
+		fprintf(fp, " <%.*s>", (int)tag_len, tag);
+
+	/* style hint */
+	switch (fye->type) {
+	default:
+		break;
+	case FYET_DOCUMENT_START:
+		if (!fy_document_event_is_implicit(fye))
+			fprintf(fp, " ---");
+		break;
+	case FYET_DOCUMENT_END:
+		if (!fy_document_event_is_implicit(fye))
+			fprintf(fp, " ...");
+		break;
+	case FYET_MAPPING_START:
+		break;
+	case FYET_SEQUENCE_START:
+		break;
+	case FYET_SCALAR:
+		style = fy_token_scalar_style(fye->scalar.value);
+		switch (style) {
+		case FYSS_ANY:
+			fprintf(fp, " !");
+			break;
+		case FYSS_PLAIN:
+			fprintf(fp, " :");
+			break;
+		case FYSS_SINGLE_QUOTED:
+			fprintf(fp, " '");
+			break;
+		case FYSS_DOUBLE_QUOTED:
+			fprintf(fp, " \"");
+			break;
+		case FYSS_LITERAL:
+			fprintf(fp, " |");
+			break;
+		case FYSS_FOLDED:
+			fprintf(fp, " >");
+			break;
+		default:
+			break;
+		}
+		break;
+	case FYET_ALIAS:
+		break;
+	}
+
+	/* content */
+	switch (fye->type) {
+	default:
+		break;
+	case FYET_SCALAR:
+		text = fy_token_get_text(fye->scalar.value, &text_len);
+		for (p = (const uint8_t *)text; text_len > 0; p += w, text_len -= (size_t)w) {
+
+			/* get width from the first octet */
+			w = (p[0] & 0x80) == 0x00 ? 1 :
+			    (p[0] & 0xe0) == 0xc0 ? 2 :
+			    (p[0] & 0xf0) == 0xe0 ? 3 :
+			    (p[0] & 0xf8) == 0xf0 ? 4 : 0;
+
+			/* error, clip it */
+			if ((size_t)w > text_len)
+				break;
+
+			/* initial value */
+			c = p[0] & (0xff >> w);
+			for (i = 1; i < w; i++) {
+				if ((p[i] & 0xc0) != 0x80)
+					break;
+				c = (c << 6) | (p[i] & 0x3f);
+			}
+
+			/* check for validity */
+			if ((w == 4 && c < 0x10000) ||
+			    (w == 3 && c <   0x800) ||
+			    (w == 2 && c <    0x80) ||
+			    (c >= 0xd800 && c <= 0xdfff) || c >= 0x110000)
+				break;
+
+			switch (c) {
+			case '\\':
+				fprintf(fp, "\\\\");
+				break;
+			case '\0':
+				fprintf(fp, "\\0");
+				break;
+			case '\b':
+				fprintf(fp, "\\b");
+				break;
+			case '\f':
+				fprintf(fp, "\\f");
+				break;
+			case '\n':
+				fprintf(fp, "\\n");
+				break;
+			case '\r':
+				fprintf(fp, "\\r");
+				break;
+			case '\t':
+				fprintf(fp, "\\t");
+				break;
+			case '\a':
+				fprintf(fp, "\\a");
+				break;
+			case '\v':
+				fprintf(fp, "\\v");
+				break;
+			case '\x1b':
+				fprintf(fp, "\\e");
+				break;
+			case 0x85:
+				fprintf(fp, "\\N");
+				break;
+			case 0xa0:
+				fprintf(fp, "\\_");
+				break;
+			case 0x2028:
+				fprintf(fp, "\\L");
+				break;
+			case 0x2029:
+				fprintf(fp, "\\P");
+				break;
+			default:
+				if ((c >= 0x01 && c <= 0x1f) || c == 0x7f ||	/* C0 */
+				    (c >= 0x80 && c <= 0x9f))			/* C1 */
+					fprintf(fp, "\\x%02x", c);
+				else
+					fprintf(fp, "%.*s", w, p);
+				break;
+			}
+		}
+		break;
+	case FYET_ALIAS:
+		alias = fy_token_get_text(fye->alias.anchor, &alias_len);
+		fprintf(fp, " %s%.*s", "*", (int)alias_len, alias);
+		break;
+	}
+
+	mbuf = fy_memstream_close(fyms, &msize);
+	return mbuf;
+}
+
 struct fy_eventp *
 fy_document_iterator_eventp_alloc(struct fy_document_iterator *fydi)
 {
@@ -1096,3 +1425,18 @@ void fy_document_iterator_event_free(struct fy_document_iterator *fydi, struct f
 
 	fy_document_iterator_eventp_recycle(fydi, fyep);
 }
+
+const char *
+fy_event_get_comments(struct fy_event *fye)
+{
+	struct fy_token *fyt;
+
+	if (!fye)
+		return NULL;
+
+	fyt = fy_event_get_token(fye);
+	if (!fyt)
+		return NULL;
+
+	return fy_token_get_comments(fyt);
+}
